@@ -11,6 +11,7 @@ ldns_dnssec_rrs_new()
 {
 	ldns_dnssec_rrs *new_rrs;
 	new_rrs = LDNS_MALLOC(ldns_dnssec_rrs);
+        if(!new_rrs) return NULL;
 	new_rrs->rr = NULL;
 	new_rrs->next = NULL;
 	return new_rrs;
@@ -96,6 +97,7 @@ ldns_dnssec_rrsets_new()
 {
 	ldns_dnssec_rrsets *new_rrsets;
 	new_rrsets = LDNS_MALLOC(ldns_dnssec_rrsets);
+        if(!new_rrsets) return NULL;
 	new_rrsets->rrs = NULL;
 	new_rrsets->type = 0;
 	new_rrsets->signatures = NULL;
@@ -318,7 +320,10 @@ ldns_dnssec_name_new_frm_rr(ldns_rr *rr)
 	ldns_dnssec_name *new_name = ldns_dnssec_name_new();
 
 	new_name->name = ldns_rr_owner(rr);
-	ldns_dnssec_name_add_rr(new_name, rr);
+	if(ldns_dnssec_name_add_rr(new_name, rr) != LDNS_STATUS_OK) {
+		ldns_dnssec_name_free(new_name);
+		return NULL;
+	}
 
 	return new_name;
 }
@@ -456,7 +461,7 @@ ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
 	} else if (typecovered == LDNS_RR_TYPE_NSEC ||
 			 typecovered == LDNS_RR_TYPE_NSEC3) {
 		if (name->nsec_signatures) {
-			ldns_dnssec_rrs_add_rr(name->nsec_signatures, rr);
+			result = ldns_dnssec_rrs_add_rr(name->nsec_signatures, rr);
 		} else {
 			name->nsec_signatures = ldns_dnssec_rrs_new();
 			name->nsec_signatures->rr = rr;
@@ -514,15 +519,6 @@ ldns_dnssec_zone_find_rrset(ldns_dnssec_zone *zone,
 	}
 }
 
-static inline void
-print_indent(FILE *out, int c)
-{
-	int i;
-	for (i=0; i<c; i++) {
-		fprintf(out, "    ");
-	}
-}
-
 void
 ldns_dnssec_name_print_soa(FILE *out, ldns_dnssec_name *name, bool show_soa)
 {
@@ -555,6 +551,7 @@ ldns_dnssec_zone *
 ldns_dnssec_zone_new()
 {
 	ldns_dnssec_zone *zone = LDNS_MALLOC(ldns_dnssec_zone);
+        if(!zone) return NULL;
 	zone->soa = NULL;
 	zone->names = NULL;
 
@@ -653,6 +650,7 @@ ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone, ldns_rr *rr)
 
 	if (!zone->names) {
 		zone->names = ldns_rbtree_create(ldns_dname_compare_v);
+                if(!zone->names) return LDNS_STATUS_MEM_ERR;
 	}
 
 	/* we need the original of the hashed name if this is
@@ -674,13 +672,18 @@ ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone, ldns_rr *rr)
 	if (!cur_node) {
 		/* add */
 		cur_name = ldns_dnssec_name_new_frm_rr(rr);
+                if(!cur_name) return LDNS_STATUS_MEM_ERR;
 		cur_node = LDNS_MALLOC(ldns_rbnode_t);
+                if(!cur_node) {
+                        ldns_dnssec_name_free(cur_name);
+                        return LDNS_STATUS_MEM_ERR;
+                }
 		cur_node->key = ldns_rr_owner(rr);
 		cur_node->data = cur_name;
-		ldns_rbtree_insert(zone->names, cur_node);
+		(void)ldns_rbtree_insert(zone->names, cur_node);
 	} else {
 		cur_name = (ldns_dnssec_name *) cur_node->data;
-		ldns_dnssec_name_add_rr(cur_name, rr);
+		result = ldns_dnssec_name_add_rr(cur_name, rr);
 	}
 
 	if (result != LDNS_STATUS_OK) {
@@ -784,9 +787,9 @@ ldns_dnssec_zone_add_empty_nonterminals(ldns_dnssec_zone *zone)
 		 * label in the current name (counting from the end)
 		 */
 		for (i = 1; i < next_label_count - soa_label_count; i++) {
-			lpos = cur_label_count - next_label_count + i;
+			lpos = (int)cur_label_count - (int)next_label_count + (int)i;
 			if (lpos >= 0) {
-				l1 = ldns_dname_label(cur_name, lpos);
+				l1 = ldns_dname_label(cur_name, (uint8_t)lpos);
 			} else {
 				l1 = NULL;
 			}
@@ -814,7 +817,7 @@ ldns_dnssec_zone_add_empty_nonterminals(ldns_dnssec_zone *zone)
 				}
 				new_node->key = new_name->name;
 				new_node->data = new_name;
-				ldns_rbtree_insert(zone->names, new_node);
+				(void)ldns_rbtree_insert(zone->names, new_node);
 			}
 			ldns_rdf_deep_free(l1);
 			ldns_rdf_deep_free(l2);
