@@ -267,8 +267,10 @@ ldns_sock_wait(int sockfd, struct timeval timeout, int write)
 {
 	fd_set fds;
 	int ret;
+#ifndef S_SPLINT_S
 	FD_ZERO(&fds);
 	FD_SET(FD_SET_T sockfd, &fds);
+#endif
 	if(write)
 		ret = select(sockfd+1, NULL, &fds, NULL, &timeout);
 	else
@@ -283,7 +285,7 @@ ldns_sock_wait(int sockfd, struct timeval timeout, int write)
 }
 
 ldns_status
-ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage *to, 
+ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage *to,
 		socklen_t tolen, struct timeval timeout, size_t *answer_size)
 {
 	int sockfd;
@@ -297,12 +299,25 @@ ldns_udp_send(uint8_t **result, ldns_buffer *qbin, const struct sockaddr_storage
 
 	/* wait for an response*/
 	if(!ldns_sock_wait(sockfd, timeout, 0)) {
+#ifndef USE_WINSOCK
 		close(sockfd);
+#else
+                closesocket(sockfd);
+#endif
 		return LDNS_STATUS_NETWORK_ERR;
 	}
-	
+
+        /* set to nonblocking, so if the checksum is bad, it becomes
+         * an EGAIN error and the ldns_udp_send function does not block,
+         * but returns a 'NETWORK_ERROR' much like a timeout. */
+        ldns_sock_nonblock(sockfd);
+
 	answer = ldns_udp_read_wire(sockfd, answer_size, NULL, NULL);
+#ifndef USE_WINSOCK
 	close(sockfd);
+#else
+        closesocket(sockfd);
+#endif
 
 	if (*answer_size == 0) {
 		/* oops */
@@ -335,7 +350,7 @@ int
 ldns_udp_connect(const struct sockaddr_storage *to, struct timeval ATTR_UNUSED(timeout))
 {
 	int sockfd;
-	
+
 	if ((sockfd = socket((int)((struct sockaddr*)to)->sa_family, SOCK_DGRAM, 
 					IPPROTO_UDP)) 
 			== -1) {
@@ -349,7 +364,7 @@ ldns_tcp_connect(const struct sockaddr_storage *to, socklen_t tolen,
 		struct timeval timeout)
 {
 	int sockfd;
-	
+
 	if ((sockfd = socket((int)((struct sockaddr*)to)->sa_family, SOCK_STREAM, 
 					IPPROTO_TCP)) == -1) {
 		return 0;
@@ -383,7 +398,11 @@ ldns_tcp_connect(const struct sockaddr_storage *to, socklen_t tolen,
 		socklen_t len = (socklen_t)sizeof(error);
 
 		if(!ldns_sock_wait(sockfd, timeout, 1)) {
+#ifndef USE_WINSOCK
 			close(sockfd);
+#else
+			closesocket(sockfd);
+#endif
 			return 0;
 		}
 
@@ -437,6 +456,7 @@ ldns_tcp_send_query(ldns_buffer *qbin, int sockfd,
 
 	/* add length of packet */
 	sendbuf = LDNS_XMALLOC(uint8_t, ldns_buffer_position(qbin) + 2);
+	if(!sendbuf) return 0;
 	ldns_write_uint16(sendbuf, ldns_buffer_position(qbin));
 	memcpy(sendbuf + 2, ldns_buffer_export(qbin), ldns_buffer_position(qbin));
 
@@ -474,7 +494,7 @@ uint8_t *
 ldns_udp_read_wire(int sockfd, size_t *size, struct sockaddr_storage *from,
 		socklen_t *fromlen)
 {
-	uint8_t *wire;
+	uint8_t *wire, *wireout;
 	ssize_t wire_size;
 
 	wire = LDNS_XMALLOC(uint8_t, LDNS_MAX_PACKETLEN);
@@ -494,9 +514,10 @@ ldns_udp_read_wire(int sockfd, size_t *size, struct sockaddr_storage *from,
 	}
 
 	*size = (size_t)wire_size;
-	wire = LDNS_XREALLOC(wire, uint8_t, (size_t)wire_size);
+	wireout = LDNS_XREALLOC(wire, uint8_t, (size_t)wire_size);
+	if(!wireout) LDNS_FREE(wire);
 
-	return wire;
+	return wireout;
 }
 
 uint8_t *
@@ -518,18 +539,24 @@ ldns_tcp_read_wire_timeout(int sockfd, size_t *size, struct timeval timeout)
 			LDNS_FREE(wire);
 			return NULL;
 		}
-		bytes = recv(sockfd, (void*)wire, 2, 0);
-		if (bytes == -1 || bytes == 0) {
+		rc = recv(sockfd, (void*) (wire + bytes), 
+				(size_t) (2 - bytes), 0);
+		if (rc == -1 || rc == 0) {
 			*size = 0;
 			LDNS_FREE(wire);
 			return NULL;
 		}
+                bytes += rc;
 	}
 
 	wire_size = ldns_read_uint16(wire);
 	
 	LDNS_FREE(wire);
 	wire = LDNS_XMALLOC(uint8_t, wire_size);
+	if (!wire) {
+		*size = 0;
+		return NULL;
+	}
 	bytes = 0;
 
 	while (bytes < (ssize_t) wire_size) {
@@ -566,18 +593,24 @@ ldns_tcp_read_wire(int sockfd, size_t *size)
 	}
 	
 	while (bytes < 2) {
-		bytes = recv(sockfd, (void*)wire, 2, 0);
-		if (bytes == -1 || bytes == 0) {
+		rc = recv(sockfd, (void*) (wire + bytes), 
+				(size_t) (2 - bytes), 0);
+		if (rc == -1 || rc == 0) {
 			*size = 0;
 			LDNS_FREE(wire);
 			return NULL;
 		}
+                bytes += rc;
 	}
 
 	wire_size = ldns_read_uint16(wire);
 	
 	LDNS_FREE(wire);
 	wire = LDNS_XMALLOC(uint8_t, wire_size);
+	if (!wire) {
+		*size = 0;
+		return NULL;
+	}
 	bytes = 0;
 
 	while (bytes < (ssize_t) wire_size) {
@@ -612,7 +645,11 @@ ldns_tcp_send(uint8_t **result,  ldns_buffer *qbin, const struct sockaddr_storag
 	}
 
 	answer = ldns_tcp_read_wire_timeout(sockfd, answer_size, timeout);
+#ifndef USE_WINSOCK
 	close(sockfd);
+#else
+	closesocket(sockfd);
+#endif
 
 	if (*answer_size == 0) {
 		/* oops */
@@ -620,8 +657,11 @@ ldns_tcp_send(uint8_t **result,  ldns_buffer *qbin, const struct sockaddr_storag
 	}
 
 	/* resize accordingly */
-	answer = (uint8_t*)LDNS_XREALLOC(answer, uint8_t *, (size_t)*answer_size);
-	*result = answer;
+	*result = (uint8_t*)LDNS_XREALLOC(answer, uint8_t *, (size_t)*answer_size);
+        if(!*result) {
+                LDNS_FREE(answer);
+                return LDNS_STATUS_MEM_ERR;
+        }
 	return LDNS_STATUS_OK;
 }
 
@@ -738,7 +778,7 @@ ldns_axfr_start(ldns_resolver *resolver, ldns_rdf *domain, ldns_rr_class class)
                 return LDNS_STATUS_ADDRESS_ERR;
         }
         /* For AXFR, we have to make the connection ourselves */
-        /* try all nameservers (which usually would mean v4 fallback if 
+        /* try all nameservers (which usually would mean v4 fallback if
          * @hostname is used */
         for (ns_i = 0;
              ns_i < ldns_resolver_nameserver_count(resolver) &&
@@ -747,8 +787,8 @@ ldns_axfr_start(ldns_resolver *resolver, ldns_rdf *domain, ldns_rr_class class)
 	        ns = ldns_rdf2native_sockaddr_storage(
 	        	resolver->_nameservers[ns_i],
 			ldns_resolver_port(resolver), &ns_len);
-	
-		resolver->_socket = ldns_tcp_connect(ns, (socklen_t)ns_len, 
+
+		resolver->_socket = ldns_tcp_connect(ns, (socklen_t)ns_len,
 				ldns_resolver_timeout(resolver));
 	}
 
@@ -765,26 +805,70 @@ ldns_axfr_start(ldns_resolver *resolver, ldns_rdf *domain, ldns_rr_class class)
 		                            ldns_resolver_tsig_keydata(resolver),
 		                            300, ldns_resolver_tsig_algorithm(resolver), NULL);
 		if (status != LDNS_STATUS_OK) {
+			/* RoRi: to prevent problems on subsequent calls to ldns_axfr_start
+			   we have to close the socket here! */
+#ifndef USE_WINSOCK
+			close(resolver->_socket);
+#else
+			closesocket(resolver->_socket);
+#endif
+			resolver->_socket = 0;
+
 			return LDNS_STATUS_CRYPTO_TSIG_ERR;
 		}
 	}
 #endif /* HAVE_SSL */
 
-        /* Convert the query to a buffer          * Is this necessary?
+        /* Convert the query to a buffer
+         * Is this necessary?
          */
         query_wire = ldns_buffer_new(LDNS_MAX_PACKETLEN);
+        if(!query_wire) {
+                ldns_pkt_free(query);
+                LDNS_FREE(ns);
+#ifndef USE_WINSOCK
+		close(resolver->_socket);
+#else
+		closesocket(resolver->_socket);
+#endif
+		resolver->_socket = 0;
+
+                return LDNS_STATUS_MEM_ERR;
+        }
         status = ldns_pkt2buffer_wire(query_wire, query);
         if (status != LDNS_STATUS_OK) {
                 ldns_pkt_free(query);
+		ldns_buffer_free(query_wire);
                 LDNS_FREE(ns);
+
+		/* RoRi: to prevent problems on subsequent calls to ldns_axfr_start
+		    we have to close the socket here! */
+#ifndef USE_WINSOCK
+		close(resolver->_socket);
+#else
+		closesocket(resolver->_socket);
+#endif
+		resolver->_socket = 0;
+
                 return status;
         }
         /* Send the query */
-        if (ldns_tcp_send_query(query_wire, resolver->_socket, ns, 
+        if (ldns_tcp_send_query(query_wire, resolver->_socket, ns,
 				(socklen_t)ns_len) == 0) {
                 ldns_pkt_free(query);
                 ldns_buffer_free(query_wire);
                 LDNS_FREE(ns);
+
+		/* RoRi: to prevent problems on subsequent calls to ldns_axfr_start
+		         we have to close the socket here! */
+
+#ifndef USE_WINSOCK
+		close(resolver->_socket);
+#else
+		closesocket(resolver->_socket);
+#endif
+		resolver->_socket = 0;
+
                 return LDNS_STATUS_NETWORK_ERR;
         }
 
