@@ -107,7 +107,7 @@
      VAR_ROOT_HINTS = 323,
      VAR_DO_NOT_QUERY_LOCALHOST = 324,
      VAR_CACHE_MAX_TTL = 325,
-     VAR_HARDEN_DNNSEC_STRIPPED = 326,
+     VAR_HARDEN_DNSSEC_STRIPPED = 326,
      VAR_ACCESS_CONTROL = 327,
      VAR_LOCAL_ZONE = 328,
      VAR_LOCAL_DATA = 329,
@@ -152,7 +152,9 @@
      VAR_SO_RCVBUF = 368,
      VAR_EDNS_BUFFER_SIZE = 369,
      VAR_PREFETCH = 370,
-     VAR_PREFETCH_KEY = 371
+     VAR_PREFETCH_KEY = 371,
+     VAR_SO_SNDBUF = 372,
+     VAR_HARDEN_BELOW_NXDOMAIN = 373
    };
 #endif
 /* Tokens.  */
@@ -224,7 +226,7 @@
 #define VAR_ROOT_HINTS 323
 #define VAR_DO_NOT_QUERY_LOCALHOST 324
 #define VAR_CACHE_MAX_TTL 325
-#define VAR_HARDEN_DNNSEC_STRIPPED 326
+#define VAR_HARDEN_DNSSEC_STRIPPED 326
 #define VAR_ACCESS_CONTROL 327
 #define VAR_LOCAL_ZONE 328
 #define VAR_LOCAL_DATA 329
@@ -270,6 +272,8 @@
 #define VAR_EDNS_BUFFER_SIZE 369
 #define VAR_PREFETCH 370
 #define VAR_PREFETCH_KEY 371
+#define VAR_SO_SNDBUF 372
+#define VAR_HARDEN_BELOW_NXDOMAIN 373
 
 
 
@@ -286,7 +290,7 @@ typedef union YYSTYPE
 
 
 /* Line 1676 of yacc.c  */
-#line 290 "util/configparser.h"
+#line 294 "util/configparser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
