@@ -39,6 +39,9 @@
  * This file contains storage for the trust anchors for the validator.
  */
 #include "config.h"
+#include <ctype.h>
+#include "ldns/dname.h"
+#include "ldns/host2wire.h"
 #include "validator/val_anchor.h"
 #include "validator/val_sigcrypt.h"
 #include "validator/autotrust.h"
@@ -69,7 +72,7 @@ anchor_cmp(const void* k1, const void* k2)
 }
 
 struct val_anchors* 
-anchors_create()
+anchors_create(void)
 {
 	struct val_anchors* a = (struct val_anchors*)calloc(1, sizeof(*a));
 	if(!a)
